from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession

import pytest


class TestServicesSite:
    @pytest.fixture(autouse=True)
    def _api_session(self, app):
        url = app.absolute_url()
        api_session = RelativeSession(url)
        api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        api_session.headers.update({"Accept": "application/json"})
        self.api_session = api_session


class TestServicesSitesGET(TestServicesSite):
    def test_sites_get_anonymous(self, app):
        self.api_session.auth = None
        response = self.api_session.get("@sites")
        assert response.status_code == 200
        data = response.json()
        assert isinstance(data, dict)
        assert data["can_manage"] is False

    def test_sites_get_authenticated(self, app):
        response = self.api_session.get("@sites")
        assert response.status_code == 200
        data = response.json()
        assert isinstance(data, dict)
        assert data["can_manage"] is True

    def test_sites_get_distributions(self, app):
        response = self.api_session.get("@sites")
        data = response.json()
        distributions = data["distributions"]
        assert isinstance(distributions, list)
        assert len(distributions) == 2
        assert distributions[0]["name"] == "default"
        assert distributions[0]["title"] == "Plone Site"

    def test_sites_get_sites(self, app):
        response = self.api_session.get("@sites")
        data = response.json()
        sites = data["sites"]
        assert isinstance(sites, list)
        assert len(sites) == 1
        assert sites[0]["id"] == "plone"
        assert sites[0]["needs_upgrade"] is False


class TestServicesSitesPOST(TestServicesSite):
    @pytest.fixture(autouse=True)
    def _answers(self):
        self.answers = {
            "site_id": "PloneBrasil",
            "title": "Plone site in Brazil",
            "description": "A simple Plone Site",
            "default_language": "pt-br",
            "portal_timezone": "America/Sao_Paulo",
            "setup_content": True,
        }

    @pytest.mark.parametrize(
        "distribution_name",
        [
            "default",
            "classic",
        ],
    )
    def test_sites_create_success(self, app, distribution_name):
        response = self.api_session.post(
            f"@sites/{distribution_name}", json=self.answers
        )
        assert response.status_code == 200
        data = response.json()
        assert data["id"] == "PloneBrasil"
        assert data["@type"] == "Plone Site"

    @pytest.mark.parametrize(
        "distribution_name",
        [
            "someStrangeName",
            "404",
        ],
    )
    def test_sites_create_failure(self, app, distribution_name):
        """Test POST with non-existing distribution names."""
        response = self.api_session.post(
            f"@sites/{distribution_name}", json=self.answers
        )
        assert response.status_code == 404
        data = response.json()
        assert data["message"].startswith("Resource not found:")



from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing.zope import WSGI_SERVER_FIXTURE


class DistributionLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.distribution
        import plone.volto

        self.loadZCML(package=plone.distribution)
        self.loadZCML(package=plone.volto)


FIXTURE = DistributionLayer()


INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="DistributionLayer:IntegrationTesting",
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, WSGI_SERVER_FIXTURE),
    name="DistributionLayer:FunctionalTesting",
)



from pathlib import Path
from plone.dexterity.interfaces import IDexterityContent
from plone.distribution.exportimport.interfaces import IDistributionBlobsMarker
from plone.namedfile.interfaces import INamedFileField
from plone.namedfile.interfaces import INamedImageField
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxfields import DefaultFieldSerializer
from zope.component import adapter
from zope.globalrequest import getRequest
from zope.interface import implementer

import logging


logger = logging.getLogger(__name__)


@adapter(INamedFileField, IDexterityContent, IDistributionBlobsMarker)
@implementer(IFieldSerializer)
class DistributionFileFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        namedfile = self.field.get(self.context)
        if namedfile is None:
            return None
        blob_path = export_blob(
            self.context.UID(), self.field.__name__, namedfile.filename, namedfile.data
        )
        result = {
            "filename": namedfile.filename,
            "content-type": namedfile.contentType,
            "size": namedfile.getSize(),
            "blob_path": blob_path,
        }
        return json_compatible(result)


@adapter(INamedImageField, IDexterityContent, IDistributionBlobsMarker)
@implementer(IFieldSerializer)
class DistributionImageFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        namedfile = self.field.get(self.context)
        if namedfile is None:
            return None
        blob_path = export_blob(
            self.context.UID(), self.field.__name__, namedfile.filename, namedfile.data
        )
        width, height = namedfile.getImageSize()
        result = {
            "filename": namedfile.filename,
            "content-type": namedfile.contentType,
            "size": namedfile.getSize(),
            "width": width,
            "height": height,
            "blob_path": blob_path,
        }
        return json_compatible(result)


def export_blob(uid, fieldname, filename, data):
    """Store blob data in a way that makes it easier to edit by hand:
    <distribution>/content/blobs/<uid>-<fieldname>/<filename>"""
    unique_dir_name_for_file = f"{uid}-{fieldname}"
    request = getRequest()
    distribution_directory = Path(request["distribution_directory"])
    target_directory = distribution_directory / "blobs" / unique_dir_name_for_file
    if not target_directory.exists():
        target_directory.mkdir(parents=True)
    target_file = target_directory / filename
    with open(target_file, "wb") as f:
        f.write(data)
    return f"blobs/{unique_dir_name_for_file}/{filename}"



from App.config import getConfiguration
from collective.exportimport import config
from collective.exportimport import import_content
from logging import getLogger
from pathlib import Path
from plone import api
from plone.distribution.api import distribution as dist_api
from plone.distribution.exportimport.interfaces import IDistributionBlobsMarker
from Products.Five import BrowserView
from zope.interface import alsoProvides


logger = getLogger(__name__)


class ImportAll(BrowserView):
    def __call__(self, path=None):
        request = self.request
        if not path and not request.form.get("form.submitted", False):
            return self.index()
        elif path:
            # path the config that is usually set via env-variables
            config.CENTRAL_DIRECTORY = str(path)
            import_content.BLOB_HOME = str(path)
        else:
            # Fallback to default (e.g. var/instance/import)
            cfg = getConfiguration()
            path = Path(cfg.clienthome) / "import"

        view = api.content.get_view("import_content", self.context, request)
        request.form["form.submitted"] = True

        # Add content
        request.form["commit"] = 500
        view(server_file="content.json", return_json=True)

        # Update the existing portal obj using the update-strategy
        request.form["handle_existing_content"] = 2
        view(server_file="portal.json", return_json=True)

        other_imports = [
            "relations",
            "members",
            "translations",
            "localroles",
            "ordering",
            "defaultpages",
            "discussion",
            "portlets",
            "redirects",
        ]
        for name in other_imports:
            view = api.content.get_view(f"import_{name}", self.context, request)
            importfile = path / f"{name}.json"
            if importfile.exists():
                results = view(jsonfile=importfile.read_text(), return_json=True)
                logger.info(results)
            else:
                logger.info(f"Skipping import of {name} because no file {importfile}")

        return request.response.redirect(self.context.absolute_url())


class ExportAll(BrowserView):
    def __call__(self):
        request = self.request
        if not request.form.get("form.submitted", False):
            return self.index()

        dist_name = request.form.get("distribution", False)
        if not dist_name:
            api.portal.show_message("Please select a target")
            return self.index()

        alsoProvides(self.request, IDistributionBlobsMarker)
        distribution = dist_api.get(dist_name)
        package_path = distribution.directory
        directory = package_path / "content"
        config.CENTRAL_DIRECTORY = str(directory)
        # pass the target-dir to blob-serializer via request
        request.form["distribution_directory"] = str(directory)

        # Export all content
        export_name = "export_content"
        view = api.content.get_view(export_name, self.context, request)
        # small hack to get all exportable types from the view
        view.portal_type = []
        view.path = "/".join(self.context.getPhysicalPath())
        view.depth = -1
        portal_types = [
            i["value"] for i in view.portal_types() if i["value"] != "Plone Site"
        ]
        request.form["filename"] = "content.json"
        view(portal_type=portal_types, include_blobs=2, download_to_server=True)
        logger.info("Finished {}".format(export_name))

        portal_types = ["Plone Site"]
        request.form["filename"] = "portal.json"
        view(portal_type=portal_types, include_blobs=2, download_to_server=True)
        logger.info("Finished {}".format(export_name))

        other_exports = [
            "relations",
            "members",
            "translations",
            "localroles",
            "ordering",
            "defaultpages",
            "discussion",
            "portlets",
            "redirects",
        ]
        for export in other_exports:
            export_view = api.content.get_view(
                f"export_{export}", self.context, request
            )
            request.form["form.submitted"] = True
            request.form["filename"] = f"{export}.json"
            export_view(download_to_server=True)

        logger.info("Finished export_all")
        return self.request.response.redirect(self.context.absolute_url())

    def distributions(self):
        return dist_api.get_distributions()



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""
from collective.exportimport.interfaces import IPathBlobsMarker


class IDistributionBlobsMarker(IPathBlobsMarker):
    """A marker interface to override default serializers."""



from plone import api
from plone.distribution.api import site as site_api
from Products.CMFPlone.Portal import PloneSite

import pytest


class TestApiSite:
    def test_get_sites(self, app, integration):
        sites = site_api.get_sites(app)
        # Integration test creates a Plone Site
        assert len(sites) == 1
        site = sites[0]
        assert isinstance(site, PloneSite)

    @pytest.mark.parametrize(
        "distribution_name",
        [
            "default",
            "classic",
        ],
    )
    def test_create(self, app, integration, distribution_name):
        with api.env.adopt_roles(["Manager"]):
            answers = {
                "site_id": "PloneBrasil",
                "title": "Plone site in Brazil",
                "description": "A simple Plone Site",
                "default_language": "pt-br",
                "portal_timezone": "America/Sao_Paulo",
                "setup_content": False,  # Avoid commiting a transaction
            }
            new_site = site_api.create(app, distribution_name, answers)
        sites = site_api.get_sites(app)
        assert len(sites) == 2
        site = sites[-1]
        assert isinstance(site, PloneSite)
        assert site.title == new_site.title
